<?php
include_once "Mikrobill.php";

class EasyPay
{
    const PLUGIN_ID = 115;

    private $connection;
    private $mikroBill;
    private $token;

    function __construct($connection, $token)
    {
        $this->connection = $connection;
        $this->mikroBill = new Mikrobill($connection);
        $this->token = $token;
    }

	function InitStrings(){
	  
	  $FileName=dirname(__DIR__) . "../../kassa/lng/".$GLOBALS['EasyPayAM_Language'].".txt";
		
	  if (!file_exists($FileName)){
		$FileName=dirname(__DIR__) . "../../kassa/lng/hay.txt";
	  }
	  
	  $file = fopen($FileName, 'r');

	  $strings=array();
	  if ($file) {
		while (($line = fgets($file)) !== false) {
		  $strings[]=trim($line);
		}
		fclose($file);
	  } 
	  
	  $GLOBALS['strings']=$strings;
	  
	}

    /**
     * @param CheckParams $params
     * @return array
     */
    function check($params)
    {
        try {
			$this->InitStrings();
			$strings=$GLOBALS['strings'];
			
            $params->check($this->token);
            $user = $this->mikroBill->findUserByLogin($params->getFirstInput(),false,0);
			
			// 1 - Абонент
			// 28 - Договор
			// 3 - Баланс
			// 2 - Тариф
			// 57 - Адрес подключения
			
            $propertyList = [
                ['key' => $strings[1], 'value' => (string)$user->name],
				['key' => $strings[57], 'value' => (string)$user->address],
                ['key' => $strings[28], 'value' => (string)$user->login],
                ['key' => $strings[3], 'value' => (string)$user->balance . $GLOBALS['curr']],
                ['key' => $strings[2], 'value' => (string)$user->payment . $GLOBALS['curr']]
            ];
            $response = ['ResponseMessage' => 'Действие разрешено', 'ResponseCode' => 0];
            //$response['Debt'] = abs($user->balance - $user->payment);
			$response['Debt'] = $user->Recommended;
            $response['Checksum'] = md5($this->token . json_encode($propertyList,JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
            $response['PropertyList'] = $propertyList;
            return $response;
        } catch (Exception $e) {
            return ['ResponseMessage' => $e->getMessage(), 'ResponseCode' => (string)$e->getCode()];
        }
    }


    /**
     * @param PaymentParams $params
     * @return array response
     */
    function pay($params)
    {
        try {
			$this->InitStrings();
			$strings=$GLOBALS['strings'];
			
            $dtTime = date('Y-m-d\TH:i:s');
            $params->check($this->token);
            $user = $this->mikroBill->findUserByLogin($params->getFirstInput(),true,$params->amount);
            $this->mikroBill->checkPayment($params->transactID);
            $paymentId = $this->mikroBill->writeToEasyPayAmLog($user->login, $params->amount, $params->transactID, date("Y-m-d H:i:s"));
            $this->mikroBill->addMoney($user->id, EasyPay::PLUGIN_ID, '', $params->amount);
			
			// 48 - Баланс будет пополнен в течении минуты!
            $response = ['ResponseCode' => 0, 'ResponseMessage' => $strings[48]];
            $response['PropertyList'] = [['key' => 'Номер платежа', 'value' => $paymentId]];
            $response['Checksum'] = md5($this->token . $dtTime);
            $response['DtTime'] = (string)$dtTime;
            return $response;
        } catch (Exception $e) {
            return ['ResponseMessage' => $e->getMessage(), 'ResponseCode' => (string)$e->getCode()];
        }
    }


}

class CheckParams
{
    /**
     * @var string
     */
    public $checksum;
    /**
     * @var array
     */
    public $inputs;
    /**
     * @var string
     */
    public $currency;
    /**
     * @var string
     */
    public $lang;

    function __construct($params)
    {
        $this->checksum = $params['Checksum'];
        $this->inputs = $params['Inputs'];
        $this->currency = $params['Currency'];
        if (isset($params['Lang'])){$this->lang = $params['Lang'];}
    }

    function check($token)
    {
        $inputs = implode('', $this->inputs);
        $hash = md5($token . $inputs . $this->lang);
        if ($this->checksum !== $hash) {
            throw new Exception("Wrong checksum", 11);
        }
    }

    function getFirstInput()
    {
        return $this->inputs[0];
    }

}

class PaymentParams
{
    /**
     * @var array
     */
    public $inputs;
    /**
     * @var float
     */
    public $amount;
    /**
     * @var string
     */
    public $transactID;
    /**
     * @var string
     */
    public $currency;
    /**
     * @var string
     */
    public $checksum;
    /**
     * @var string
     */
    public $dtTime;


    function __construct($params)
    {
        $this->inputs = $params['Inputs'];
        $this->amount = $params['Amount'];
        $this->transactID = $params['TransactID'];
        $this->currency = $params['Currency'];
        $this->checksum = $params['Checksum'];
        $this->dtTime = $params['DtTime'];
    }

    function check($token)
    {
        $inputs = implode('', $this->inputs);
        $hash = md5($token . $inputs . $this->amount . $this->transactID);
        if ($this->checksum !== $hash) {
            throw new Exception("Wrong checksum", 11);
        }
    }

    function getFirstInput()
    {
        return $this->inputs[0];
    }


    function getTransactionDatetime()
    {
        return date('Y-m-d H:i:s', strtotime($this->dtTime));
    }
}
?>