<?php
include './Mikrobill.php';

class HayPost
{
    const PLUGIN_ID = 114;
    const PLUGIN_COMMENT = '';

    private $db;
    private $mikroBill;
	

    function __construct($connection)
    {
        $this->db = $connection;
        $this->mikroBill = new Mikrobill($connection);
    }

    function checkDept($contract, $token, $GoodToken)
    {
        try {
            $this->isValidToken($token, $GoodToken);
            $user = $this->mikroBill->findUserByContract($contract,false,0);
            $this->response([
                'Response' => '0',
                'Description' => 'data sent',
                'contractId' => $user->contract,
                'name' => $user->firstName,
                'surname' => $user->lastName,
                'tariff' => $user->payment,
                'balance' => $user->balance
            ]);

        } catch (Exception $e) {
            $this->error($e->getMessage());
        }
    }

    function pay($contract, $sum, $transactionId, $date, $token, $GoodToken)
    {
        try {
            $this->isValidToken($token, $GoodToken);
            $this->mikroBill->checkPayment($transactionId, $contract, $sum);
            $user = $this->mikroBill->findUserByContract($contract, true, $sum);
            $paymentId = $this->mikroBill->writeToOperationLog('operations_haypost', $contract, $transactionId, $sum);
            $this->mikroBill->addMoney($user->id, HayPost::PLUGIN_ID, HayPost::PLUGIN_COMMENT, $sum);

            $this->response([
                'Response' => 0,
                'Description' => 'data sent',
                'contractId' => $user->contract,
                'name' => $user->firstName,
                'surname' => $user->lastName,
                'amount' => $sum,
                'transactionId' => $transactionId,
                'paymentId' => $paymentId,
                'debt' => 0,
                'pay_date' => $date
            ]);

        } catch (Exception $e) {
            $this->error($e->getMessage());
        }
    }

    function cancel($contractId, $debt, $transactionId, $payDate, $paymentId, $GoodToken)
    {
        $this->response([
            "Response" => "0",
            "Description" => "data sent",
            "contractId" => $contractId,
            "transactionId" => $transactionId,
            "paymentId" => $paymentId,
            "debt" => $debt,
            "pay_date" => $payDate,
            "pay_cancel_date" => date('Y-m-d H:is')
        ]);
    }

    private function response($response)
    {
        header('Content-Type: application/json');
        echo json_encode(['Result' => $response], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);
    }

    public function error($message)
    {
        $this->response(
            [
                'Response' => 1,
                'Description' => $message
            ]
        );
    }

    private function isValidToken($token, $GoodToken)
    {
        if ($token != $GoodToken) {
            throw new Exception('Wrong token');
        }
        return true;
    }
}
?>
