<?php

class Mikrobill
{
    /**
     * @var $db PDO
     * */
    private $db;


    /**
     * @param PDO $connection
     */
    function __construct($connection)
    {
        $this->db = $connection;
        try {
            $this->db->exec("SET NAMES utf8");
            $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
        } catch (PDOException $e) {
            print_r($e->getMessage());
            exit;
        }
    }

    /**
     * @param float $userId Идентификатор пользователя
     * @param string $pluginId Идентификатор плагина плотежной системы в MikroBILL
     * @param string $pluginComment Имя кассира в примечании
     * @param float $amount Сумма пополнения
     */
    public function addMoney($userId, $pluginId, $pluginComment, $amount)
    {
        ///$pluginId = "ID=" . $pluginId;
		$pluginId = "Haypost";
        $uid = uniqid("", true) . uniqid("", true);
        $query = "INSERT INTO addcash2 VALUES (:userId, :amount, 0, :pluginId, 0, :uid, :comment , :date)";
        $statement = $this->db->prepare($query);
        $statement->execute([
            ':userId' => $userId,
            ':amount' => $amount,
            ':pluginId' => $pluginId,
            ':uid' => $uid,
            ':comment' => $pluginComment,
            ':date' => date("Y-m-d H:i:s")]);
        $this->writeToActionLog('ADD_MONEY', $userId, $amount, $pluginId);
        $this->refreshDataBase();
    }

    public function writeToActionLog($action, $guid, $amount, $initiator)
    {
        $dateTime = date("Y-m-d H:i:s");
        $statement = $this->db->prepare("INSERT INTO actionslog VALUES (?,?,?,?,'',?);");
        $statement->execute([$dateTime, $action, $guid, $amount, $initiator]);
    }

    public function refreshDataBase()
    {
        //$this->db->exec("INSERT INTO workparams VALUES('REFRESH_DB','')");
		$this->db->exec("INSERT INTO `refresh_db` VALUES (1);");
    }

    /**
     * @param $tableName string Имя таблицы с логом платежей
     * @param $id string уникальный идентификатор платежа
     * @return bool
     */
    public function isDuplicatePayment2($tableName, $id, $account, $pay_amount)
    {
        $statement = $this->db->prepare("SELECT `id` FROM $tableName WHERE operation_id LIKE ? AND contract LIKE ? AND sum LIKE ?;");
        $statement->execute([$id, $account, $pay_amount]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if ($result !== false) {
            return 1;
        }
        $statement = $this->db->prepare("SELECT `id` FROM $tableName WHERE operation_id LIKE ? ;");
        $statement->execute([$id]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if ($result !== false) {
            return 2;
        }
        return 3;
    }

    public function checkPayment($transactionId, $account, $sum)
    {
        $statement = $this->db->prepare("SELECT `id` FROM operations_haypost WHERE transaction_id = ? AND contract = ? AND sum = ?;");
        $statement->execute([$transactionId, $account, $sum]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if ($result) {
            throw new Exception('Платеж с даным номером уже был проведен');
        }
        return true;
    }

    /**
     * @param $login string Логин клиента в личном кабинете
     * @return MikroBillUser
     * @throws Exception
     */
    public function findUserByLogin($login)
    {
        $query = 'SELECT shortguid, pinfo, ballance, otherinfo FROM stat WHERE user_name = ?';
        $statement = $this->db->prepare($query);
        $statement->execute([$login]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if ($result === false) {
            throw new Exception('Пользователь не найден');
        }

        $balance = explode(' ', $result['ballance']);
        //$payment = explode(' ', $result['user_pay']);
        $pinfo = explode('||', $result['pinfo']);
		$otherinfo = explode('||', $result['otherinfo']);

        $user = new MikroBillUser();
        $user->id = $result['shortguid'];
        $user->login = $login;
        $user->contract = $pinfo[5];
        $user->fullName = $pinfo[3];
        $user->phone = $pinfo[0];
        $user->address = $pinfo[4];
        $user->balance = (float)$balance[0];
        $user->payment = (float)$otherinfo[74];
        $user->firstName = $pinfo[18];
        $user->lastName = $pinfo[17];
        $user->surName = $pinfo[19];
        return $user;
    }

    public function findUserByContract($contract, $IsPay, $sum)
    {
        $query = 'SELECT shortguid, pinfo, ballance, user_name, otherinfo FROM stat WHERE contract = ? AND `isarchived` = 0';
        $statement = $this->db->prepare($query);
        $statement->execute([$contract]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if (!$result) {
			if ($IsPay){
				$stm = $connection->prepare("INSERT INTO `unknown_payments` (`moneytime`,`cash`,`pay_type`,`comment`) VALUES (?,?,21,?);");
				$stm->execute([time(), $sum, $contract]);
			}
            throw new Exception('Пользователь не найден');
        }

        $balance = explode(' ', $result['ballance']);
        //$payment = explode(' ', $result['user_pay']);
        $pinfo = explode('||', $result['pinfo']);
		$otherinfo = explode('||', $result['otherinfo']);

        $user = new MikroBillUser();
        $user->id = $result['shortguid'];
        $user->login = $result['user_name'];
        $user->contract = $pinfo[5];
        $user->fullName = $pinfo[3];
        $user->phone = $pinfo[0];
        $user->address = $pinfo[4];
        $user->balance = (float)$balance[0];
        $user->payment = (float)$otherinfo[74];
        $user->firstName = $pinfo[18];
        $user->lastName = $pinfo[17];
        $user->surName = $pinfo[19];
        return $user;
    }

    public function writeToOperationLog($operationLog, $userLogin, $operationId, $amount)
    {
        $query = "INSERT INTO $operationLog (`contract`,`sum`,`transaction_id`,`date`) VALUES (?,?,?,?)";
        $statement = $this->db->prepare($query);
        $statement->execute([$userLogin, $amount, $operationId, date("Y-m-d H:i:s")]);
        if ($id = $this->db->lastInsertId()) {
            return $id;
        }
        throw new Exception('Не удалось записать в лог операций');
    }
}

class MikroBillUser
{
    public $id;
    public $login;
    public $contract;
    public $name;
    public $address;
    public $phone;
    public $balance;
    public $payment;
    public $firstName;
    public $lastName;
    public $surName;
}
?>