<?php

class Mikrobill
{
    /**
     * @var $db PDO
     * */
    private $db;


    /**
     * @param PDO $connection
     */
    function __construct($connection)
    {
        $this->db = $connection;
        $this->db->exec("SET NAMES utf8");
        $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    }

    /**
     * @param float $userId Идентификатор пользователя
     * @param string $pluginId Идентификатор плагина плотежной системы в MikroBILL
     * @param string $pluginComment Имя кассира в примечании
     * @param float $amount Сумма пополнения
     */
    public function addMoney($userId, $pluginId, $pluginComment, $amount)
    {
        $pluginStr = "ID=$pluginId";
        $uid = uniqid("", true) . uniqid("", true);
        $query = "INSERT INTO addcash2 VALUES (:userId, :amount, 0, :pluginId, 0, :uid, :comment , :date)";
        $statement = $this->db->prepare($query);
        $statement->execute([
            ':userId' => $userId,
            ':amount' => $amount,
            ':pluginId' => 'IPAYUA~~'.$pluginId,
            ':uid' => $uid,
            ':comment' => $pluginComment,
            ':date' => date("Y-m-d H:i:s")]);
        $this->writeToActionLog('ADD_MONEY', $userId, $amount, $pluginId);
        $this->refreshDataBase();
    }

    public function writeToActionLog($action, $guid, $amount, $initiator)
    {
        $dateTime = date("Y-m-d H:i:s");
        $statement = $this->db->prepare("INSERT INTO actionslog VALUES (?,?,?,?,'',?);");
        $statement->execute([$dateTime, $action, $guid, $amount, $initiator]);
    }

    public function refreshDataBase()
    {
        $this->db->exec("INSERT INTO refresh_db VALUES(1)");
    }

    public function isDuplicatePayment($id, $table)
    {
        $stm = $this->db->prepare("SELECT id FROM $table WHERE operation_id = ?");
        $stm->execute([$id]);
        $result = $stm->fetch(PDO::FETCH_ASSOC);
        if ($result) {
            return true;
        }
        return false;
    }

    public function findPayment($id)
    {
        $stm = $this->db->prepare("SELECT * FROM operations_sistema WHERE operation_id = ?");
        $stm->execute([$id]);
        return $stm->fetch(PDO::FETCH_ASSOC);
    }


    /**
     * @param $login string Логин клиента в личном кабинете
     * @return MikroBillUser
     * @throws Exception
     */
    public function findUserByLogin($login)
    {
        $query = 'SELECT shortguid, pinfo, ballance, user_pay, user_name FROM stat LEFT JOIN tarifs USING(tarif_guid) WHERE user_name = ? OR contract = ?';
        $statement = $this->db->prepare($query);
        $statement->execute([$login, $login]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if (!$result) {
            throw new Exception('Account not fount', -40);
        }

        $balance = explode(' ', $result['ballance']);
        $payment = explode(' ', $result['user_pay']);
        $pinfo = explode('||', $result['pinfo']);

        $user = new MikroBillUser();
        $user->id = $result['shortguid'];
        $user->login = $result['user_name'];
        $user->contract = $pinfo[5];
        $user->name = $pinfo[3];
        $user->phone = $pinfo[0];
        $user->address = $pinfo[4];
        $user->balance = (float)$balance[0];
        $user->payment = (float)$payment[0];
        $user->firstName = $pinfo[18];
        $user->lastName = $pinfo[17];
        $user->surName = $pinfo[19];
        return $user;
    }

    public function findUserById($id, $sum)
    {
        $query = 'SELECT shortguid, pinfo, ballance,user_name, user_pay FROM stat LEFT JOIN tarifs USING(tarif_guid) WHERE shortguid = ?';
        $statement = $this->db->prepare($query);
        $statement->execute([$id]);
        $result = $statement->fetch(PDO::FETCH_ASSOC);
        if (!$result) {
			$stm = $this->db->prepare("INSERT INTO `unknown_payments` (`moneytime`,`cash`,`pay_type`,`comment`) VALUES (?,?,47,?);");
			$stm->execute([time(), $sum, $id]);
            throw new Exception('Account not fount', -40);
        }

        $balance = explode(' ', $result['ballance']);
        $payment = explode(' ', $result['user_pay']);
        $pinfo = explode('||', $result['pinfo']);

        $user = new MikroBillUser();
        $user->id = $result['shortguid'];
        $user->login = $result['user_name'];
        $user->contract = $pinfo[5];
        $user->phone = $pinfo[0];
        $user->name = $pinfo[3];
        $user->address = $pinfo[4];
        $user->balance = (float)$balance[0];
        $user->payment = (float)$payment[0];
        $user->firstName = $pinfo[18];
        $user->lastName = $pinfo[17];
        $user->surName = $pinfo[19];
        return $user;
    }

    public function writeToIpayOperationLog($login, $amount, $operationId, $status)
    {
        $query = "INSERT INTO operations_ipayua (`sum`,`operation_id`,`contract`, `date`, `status`) VALUES (?,?,?,?,?)";
        $statement = $this->db->prepare($query);
        $statement->execute([$amount, $operationId, $login, date("Y-m-d H:i:s"), $status]);
        return $this->db->lastInsertId();
    }
	
	public function testDbConnection(){
		try{
        $query = "SELECT 12 ";
        $stm = $this->db->prepare($query);
        $stm->execute();
        return $stm->fetch();
		}catch (Exception $e){
			var_dump($e);
		}
    }
	
}

class MikroBillUser
{
    public $id;
    public $login;
    public $contract;
    public $name;
    public $address;
    public $phone;
    public $balance;
    public $payment;
    public $firstName;
    public $lastName;
    public $surName;
}

class Company
{
    public $id;
    public $name;
    public $phone;
    public $account;
    public $okpo;
    public $mfo;
}