<?php
include_once "Mikrobill.php";

class MobiDRAM
{
    const PLUGIN_ID = 115;

    private $connection;
    private $mikroBill;
    private $token;

    function __construct($connection, $token)
    {
        $this->connection = $connection;
        $this->mikroBill = new Mikrobill($connection);
        $this->token = $token;
    }

    /**
     * @param CheckParams $params
     * @return array
     */
    function check($params)
    {
        try {
            $params->check($this->token);
            $user = $this->mikroBill->findUserByLogin($params->getFirstInput(),false,0);
            $propertyList = [
                ['key' => 'Абонент', 'value' => (string)$user->name],
                ['key' => 'Договор', 'value' => (string)$user->login],
                ['key' => 'Баланс', 'value' => (string)$user->balance],
                ['key' => 'Тариф', 'value' => (string)$user->payment]
            ];
            $response = ['ResponseMessage' => 'Действие разрешено', 'ResponseCode' => 0];
            //$response['Debt'] = abs($user->balance - $user->payment);
			$response['Debt'] = $user->Recommended;
            $response['Checksum'] = md5($this->token . json_encode($propertyList,JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
            $response['PropertyList'] = $propertyList;
            return $response;
        } catch (Exception $e) {
            return ['ResponseMessage' => $e->getMessage(), 'ResponseCode' => (string)$e->getCode()];
        }
    }


    /**
     * @param PaymentParams $params
     * @return array response
     */
    function pay($params)
    {
        try {
            $dtTime = date('Y-m-d\TH:i:s');
            $params->check($this->token);
            $user = $this->mikroBill->findUserByLogin($params->getFirstInput(),true,$params->amount);
            $this->mikroBill->checkPayment($params->transactID);
            $paymentId = $this->mikroBill->writeToEasyPayAmLog($user->login, $params->amount, $params->transactID, date("Y-m-d H:i:s"));
            $this->mikroBill->addMoney($user->id, MobiDRAM::PLUGIN_ID, 'MOBIDRAM', $params->amount);
            $response = ['ResponseCode' => 0, 'ResponseMessage' => 'Платеж принято'];
            $response['PropertyList'] = [['key' => 'Номер платежа', 'value' => $paymentId]];
            $response['Checksum'] = md5($this->token . $dtTime);
            $response['DtTime'] = (string)$dtTime;
            return $response;
        } catch (Exception $e) {
            return ['ResponseMessage' => $e->getMessage(), 'ResponseCode' => (string)$e->getCode()];
        }
    }


}

class CheckParams
{
    /**
     * @var string
     */
    public $checksum;
    /**
     * @var array
     */
    public $inputs;
    /**
     * @var string
     */
    public $currency;
    /**
     * @var string
     */
    public $lang;

    function __construct($params)
    {
        $this->checksum = $params['Checksum'];
        $this->inputs = $params['Inputs'];
        $this->currency = $params['Currency'];
        $this->lang = $params['Lang'];
    }

    function check($token)
    {
        $inputs = implode('', $this->inputs);
        $hash = md5($token . $inputs . $this->lang);
        if ($this->checksum !== $hash) {
            throw new Exception("Wrong checksum", 11);
        }
    }

    function getFirstInput()
    {
        return $this->inputs[0];
    }

}

class PaymentParams
{
    /**
     * @var array
     */
    public $inputs;
    /**
     * @var float
     */
    public $amount;
    /**
     * @var string
     */
    public $transactID;
    /**
     * @var string
     */
    public $currency;
    /**
     * @var string
     */
    public $checksum;
    /**
     * @var string
     */
    public $dtTime;


    function __construct($params)
    {
        $this->inputs = $params['Inputs'];
        $this->amount = $params['Amount'];
        $this->transactID = $params['TransactID'];
        $this->currency = $params['Currency'];
        $this->checksum = $params['Checksum'];
        $this->dtTime = $params['DtTime'];
    }

    function check($token)
    {
        $inputs = implode('', $this->inputs);
        $hash = md5($token . $inputs . $this->amount . $this->transactID);
        if ($this->checksum !== $hash) {
            throw new Exception("Wrong checksum", 11);
        }
    }

    function getFirstInput()
    {
        return $this->inputs[0];
    }


    function getTransactionDatetime()
    {
        return date('Y-m-d H:i:s', strtotime($this->dtTime));
    }
}
?>