<?php

class Sistema
{
    const PLUGIN_ID = 107;

    /**
     * @var MikroBill
     */
    private $mbApi;

    function __construct($mbApi)
    {
        $this->mbApi = $mbApi;
    }

    /**
     * @param array $params
     * @return string
     * @throws Exception
     */
    public function process($params)
    {
        switch ($params['cmd']) {
            case 'verify':
                if (!$params['account']) {
                    throw new Exception('Account not fount', -40);
                }
                return $this->verify($params['account']);
            case 'pay':
                if (!$params['account'] || !$params['sum'] || !$params['id']) {
                    throw new Exception('Incorrect incoming params', -90);
                }
                return $this->pay($params['account'], $params['sum'], $params['id']);
            case 'check':
                return $this->check($params['id']);
            default:
                throw new Exception('Action not defined', -90);
        }
    }

    /**
     * @param $paymentId
     * @return string
     * @throws Exception
     */
    private function check($paymentId)
    {
        $payment = $this->mbApi->findPayment($paymentId);
        if (!$payment) {
            throw new Exception('Transaction not found', -27);
        }
        return $this->getPayAnswer($payment['operation_id'], $payment['id'], $payment['date']);
    }

    /**
     * @param string $account
     * @param string $sum
     * @param string $paymentId
     * @return string
     * @throws Exception
     */
    private function pay($account, $sum, $paymentId)
    {
        if ($payment = $this->mbApi->findPayment($paymentId)) {
            return $this->getPayAnswer($payment['operation_id'], $payment['id'], $payment['date']);
        }
        $user = $this->mbApi->findUserByContract($account);
        $this->mbApi->addMoney($user->id, 'SISTEMA', '', $sum);
        $providerId = $this->mbApi->writeToSistemaOperationLog($user->login, $sum, $paymentId);
        return $this->getPayAnswer($paymentId, $providerId, date('Y-m-d H:i:s'));
    }

    /**
     * @param string $account
     * @return string
     * @throws Exception
     */
    private function verify($account)
    {
        if (!$user = $this->mbApi->findUserByLogin($account)) {
            throw new Exception('Account not fount', -40);
        }
        return '<?xml version="1.0" encoding="windows-1251"?>
                <response>
                    <result>21</result>
                    <text>'.sprintf('%s %s %s',$user->lastName, $user->firstName, $user->surName).'</text>
                    <comment>Account exist.</comment>
                </response>';
    }

    private function getPayAnswer($sistemaId, $providerId, $providerTime)
    {
        return '<?xml version="1.0" encoding="windows-1251"?>
                <response>
                    <result>27</result>
                    <id>' . $sistemaId . '</id>
                    <provider_id>' . $providerId . '</provider_id>
                    <provider_time>' . $providerTime . '</provider_time>
                    <comment>Transaction complete</comment>
                </response>';
    }
}